; Pre-emptive thread demo. The APU is recruited to generate
; a periodic interrupt to switch threads without having to
; manually call yield from each. This also ensures that both
; threads get equal processor time.
;
; The threads are switched every 1/60 second which results
; in somewhat choppy movement. For finer granularity, the
; DMC interrupt can be used, allowing switching at up to
; 4142 times per second. See pre-emptive_dmc.asm for example
; of this.

      .include "shell.asm"
      .include "threading.asm"
main:
      ; disable interrupt source before clearing i flag
      lda   #$40
      sta   $4017
      cli
      
      ; Fork second thread
      jsr   fork
      bcs   thread2
      
      ; start thread-switching interrupt
      lda   #$00
      sta   $4017
      
; Both these loops run simultaneously
thread:
      ldy   #150        ; delay
dly:  dey
      nop
      nop
      bne   dly
      inc   sprite0_x   ; move sprite
      jmp   thread
      
thread2:
      ldy   #255        ; delay longer
dly2: dey
      nop
      nop
      bne   dly2
      inc   sprite1_x   ; move sprite
      jmp   thread2
      
; Change thread interrupt occurs periodically
irq:  bit   $4015       ; acknowledge interrupt
      jsr   yield       ; switch threads
      rti
