; Thread handling. For clarity, only handles two threads.
; Thread 0 is the main thread. Thread 1's stack top is $80.

      .zp
current_thread:
      .ds 1       ; 0 or 1
threads:
      .ds 2       ; stack pointers for each thread
      .code

; "Fork" main thread into two threads. Main thread returns
; with carry flag clear, and second thread returns with it
; set. All other registers are the same as on entry.
fork:
      ; carry set for new thread
      sec
      
      ; save context
      php
      pha
      txa
      pha
      tya
      pha
      
      ; make copy of stack for second thread
      ldy   #$7F
clp:  lda   $180,y
      sta   $100,y
      dey
      bne   clp
      
      ; set stack pointer for second thread
      tsx
      txa
      eor   #$80
      sta   threads + 1
      
      ; restore context
      pla
      tay
      pla
      tax
      pla
      plp
      
      ; carry clear for calling thread
      clc
      
      rts
      
; Switch threads and resume where other thread last called
; yield.
yield:
      ; save context
      php
      pha
      txa
      pha
      tya
      pha
      
      ; save stack
      ldy   current_thread
      tsx
      stx   <threads,y
      
      ; switch threads
      tya
      eor   #1
      tay
      sty   current_thread
      
      ; load stack of new thread
      ldx   <threads,y
      txs
      
      ; restore context
      pla
      tay
      pla
      tax
      pla
      plp
      rts
      